/*!
 * Built with Stencil
 * By PostNord.
 */
import { awaitTopbar, en, ripple } from "../../../index";
import { Host, h } from "@stencil/core";
import { alert_info_circle, check_circle, alert_exclamation_triangle, alert_exclamation_circle, close, } from "pn-design-assets/pn-assets/icons.js";
import { translations } from "./translation";
/**
 * This component is able to fill many roles of a classic toast.
 * It has 4 different colors, a temporary state, hide button, icon and illustration support.
 *
 * The `pn-toast` will expand to its parent containers full width.
 * Either limit the parent width or set a CSS rule to your specific width if needed.
 */
export class PnToast {
    constructor() {
        this.isClosing = false;
        this.isExpanding = false;
        this.hasSlottedContent = false;
        this.heading = '';
        this.text = '';
        this.toastId = undefined;
        this.appearance = undefined;
        this.temporary = false;
        this.icon = undefined;
        this.illustration = undefined;
        this.hide = false;
        this.closable = false;
        this.language = null;
    }
    toast;
    button;
    animation;
    hostElement;
    handleHide() {
        const { matches } = window.matchMedia('(prefers-reduced-motion: reduce)');
        if (matches)
            return this.hidden.emit(this.hide);
        if (this.hide)
            this.closeToast();
        else
            this.openToast();
    }
    /** Event fired when the close button is pressed. */
    close;
    /**
     * This event is fired when the toast is fully hidden after the animation is finished.
     * Triggers from the close button or if the prop `hide` is set to true.
     **/
    hidden;
    async componentWillLoad() {
        if (this.hide)
            this.hostElement.style.height = '0';
        if (this.language === null)
            await awaitTopbar(this.hostElement);
        // Any text content at this stage means there is slotted content present.
        this.hasSlottedContent = !!this.hostElement.textContent;
    }
    componentDidLoad() {
        if (!!this.heading && !this.text && !this.hasSlottedContent)
            console.warn('The %s prop is only available if you use the %s prop or slotted content.', 'heading', 'text');
    }
    getRect(element) {
        return element.getBoundingClientRect();
    }
    openToast() {
        requestAnimationFrame(() => {
            const { scrollHeight } = this.hostElement;
            const { height } = this.getRect(this.hostElement);
            this.hostElement.style.height = `${height}px`;
            this.isExpanding = true;
            this.animateToast(true, `${height}px`, `${scrollHeight}px`);
        });
    }
    closeToast() {
        const { scrollHeight, clientHeight } = this.hostElement;
        const height = this.isExpanding ? clientHeight : scrollHeight;
        this.hostElement.style.height = `0px`;
        this.isClosing = true;
        this.animateToast(false, `${height}px`, `0px`);
    }
    animateToast(open, startHeight, endHeight) {
        this.cancelAnimations();
        this.animation = this.hostElement.animate({
            height: [startHeight, endHeight],
        }, {
            duration: 400,
            easing: 'cubic-bezier(0.6, 0, 0.2, 1)',
        });
        this.animation.onfinish = () => this.animationFinish();
        this.animation.oncancel = () => (open ? (this.isExpanding = false) : (this.isClosing = false));
    }
    animationFinish() {
        this.cancelAnimations();
        if (!this.hide)
            this.hostElement.style.height = '';
        else
            this.hidden.emit(this.hide);
        this.isClosing = false;
        this.isExpanding = false;
    }
    cancelAnimations() {
        if (this.animation)
            this.animation.cancel();
    }
    handleClick(event) {
        this.hide = true;
        this.close.emit(this.hide);
        ripple(event, this.toast);
    }
    getIcon() {
        if (this.illustration)
            return '';
        if (this.icon)
            return this.icon;
        if (this.appearance === 'success')
            return check_circle;
        if (this.appearance === 'warning')
            return alert_exclamation_triangle;
        if (this.appearance === 'error')
            return alert_exclamation_circle;
        return alert_info_circle;
    }
    getRole() {
        return this.appearance === 'error' ? 'alert' : 'status';
    }
    showIcon() {
        return !this.illustration;
    }
    showIllustration() {
        return !this.showIcon();
    }
    showTitle() {
        return (this.hasSlottedContent || !!this.text) && !!this.heading;
    }
    translate(prop) {
        return translations?.[prop]?.[this.language || en] || prop;
    }
    hideToast() {
        return this.hide && !(this.isClosing || this.isExpanding);
    }
    render() {
        return (h(Host, { key: '4d7cc11bfc2d8a2f0a1b42cf291bcf09e810d4a8' }, h("output", { key: 'a1d2e02c4936c863efba2e3b4744260096b966b0', id: this.toastId, class: "pn-toast", role: this.getRole(), "data-appearance": this.appearance, "data-temporary": this.temporary, "data-hide": this.hideToast(), ref: el => (this.toast = el) }, this.showIcon() && h("pn-icon", { key: 'b4aac661b5f62cec6205aa81374c82d259186d32', class: "pn-toast-icon", icon: this.getIcon() }), h("div", { key: 'afd8f78fab826cfe2a8d00a02158c6504f77ec3c', class: "pn-toast-content" }, this.showTitle() && h("h4", { key: '6d32326637df6723c3d7e68b84f9190b59fa5765', class: "pn-toast-heading" }, this.heading), this.text && h("p", { key: 'abdbb69a78692b81105f5b3134afdf4be1e27b55', class: "pn-toast-text" }, this.text), h("slot", { key: '45c3b143b92ec4350542d17f3a4295028d00631c' })), this.showIllustration() && h("pn-illustration", { key: 'e9569b033e296e7c7310521ca4a67517553f1355', illustration: this.illustration, width: "5.5em", height: "5.5em" }), this.closable && (h("button", { key: '8c4fb6be7e19db8f9889eb44c765f115d026cd9c', class: "pn-toast-button", type: "button", "aria-label": this.translate('HIDE'), onClick: (event) => this.handleClick(event), ref: el => (this.button = el) }, h("pn-icon", { key: '3865a38e56f96cccd7e22df16d21913bfcdd3e78', icon: close }))))));
    }
    static get is() { return "pn-toast"; }
    static get originalStyleUrls() {
        return {
            "$": ["pn-toast.scss"]
        };
    }
    static get styleUrls() {
        return {
            "$": ["pn-toast.css"]
        };
    }
    static get properties() {
        return {
            "heading": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "description",
                            "text": "Can be used together with slotted content."
                        }],
                    "text": "Set a title for the toast. Requires the `text` prop in order to work."
                },
                "attribute": "heading",
                "reflect": false,
                "defaultValue": "''"
            },
            "text": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "description",
                            "text": "Can be used together with slotted content."
                        }],
                    "text": "Set a paragraph of text for the toast."
                },
                "attribute": "text",
                "reflect": false,
                "defaultValue": "''"
            },
            "toastId": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [],
                    "text": "Give the toast a HTML ID."
                },
                "attribute": "toast-id",
                "reflect": false
            },
            "appearance": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "'' | 'success' | 'warning' | 'error'",
                    "resolved": "\"\" | \"error\" | \"success\" | \"warning\"",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "category",
                            "text": "Visual"
                        }],
                    "text": "Default color is blue, you also have the option of success (green), warning (orange) and error (red)."
                },
                "attribute": "appearance",
                "reflect": false
            },
            "temporary": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "category",
                            "text": "Visual"
                        }],
                    "text": "This will apply a shadow and border to the toast."
                },
                "attribute": "temporary",
                "reflect": false,
                "defaultValue": "false"
            },
            "icon": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "category",
                            "text": "Visual"
                        }],
                    "text": "Set an SVG icon."
                },
                "attribute": "icon",
                "reflect": false
            },
            "illustration": {
                "type": "string",
                "mutable": false,
                "complexType": {
                    "original": "string",
                    "resolved": "string",
                    "references": {}
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "category",
                            "text": "Visual"
                        }],
                    "text": "Set an SVG illustration."
                },
                "attribute": "illustration",
                "reflect": false
            },
            "hide": {
                "type": "boolean",
                "mutable": true,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "see",
                            "text": "{@link closable }"
                        }, {
                            "name": "category",
                            "text": "Features"
                        }],
                    "text": "Control the visibility of the toast. If the `closable` prop is active, the toast will set the `hide` prop on its own."
                },
                "attribute": "hide",
                "reflect": true,
                "defaultValue": "false"
            },
            "closable": {
                "type": "boolean",
                "mutable": false,
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                },
                "required": false,
                "optional": false,
                "docs": {
                    "tags": [{
                            "name": "category",
                            "text": "Features"
                        }],
                    "text": "Show close button."
                },
                "attribute": "closable",
                "reflect": false,
                "defaultValue": "false"
            },
            "language": {
                "type": "string",
                "mutable": true,
                "complexType": {
                    "original": "PnLanguages",
                    "resolved": "\"\" | \"da\" | \"en\" | \"fi\" | \"no\" | \"sv\"",
                    "references": {
                        "PnLanguages": {
                            "location": "import",
                            "path": "@/index",
                            "id": "src/index.ts::PnLanguages"
                        }
                    }
                },
                "required": false,
                "optional": true,
                "docs": {
                    "tags": [{
                            "name": "see",
                            "text": "{@link closable }"
                        }, {
                            "name": "category",
                            "text": "Features"
                        }],
                    "text": "Manually set the language, only needed if the `closable` prop is enabled."
                },
                "attribute": "language",
                "reflect": false,
                "defaultValue": "null"
            }
        };
    }
    static get states() {
        return {
            "isClosing": {},
            "isExpanding": {},
            "hasSlottedContent": {}
        };
    }
    static get events() {
        return [{
                "method": "close",
                "name": "close",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "Event fired when the close button is pressed."
                },
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                }
            }, {
                "method": "hidden",
                "name": "hidden",
                "bubbles": true,
                "cancelable": true,
                "composed": true,
                "docs": {
                    "tags": [],
                    "text": "This event is fired when the toast is fully hidden after the animation is finished.\nTriggers from the close button or if the prop `hide` is set to true."
                },
                "complexType": {
                    "original": "boolean",
                    "resolved": "boolean",
                    "references": {}
                }
            }];
    }
    static get elementRef() { return "hostElement"; }
    static get watchers() {
        return [{
                "propName": "hide",
                "methodName": "handleHide"
            }];
    }
}
//# sourceMappingURL=pn-toast.js.map
