import { EventEmitter } from '../../../stencil-public-runtime';
import { PnLanguages } from "../../../index";
import type { GridDate, CalendarView, PnDatePickerSelectEvent } from "../../../globals/date/index";
import type { Dayjs } from 'dayjs';
/**
 * The date picker allows a single or a range of dates to be selected.
 *
 * Based on the `format` prop, separators will automatically be added if you type the date.
 *
 * You can navigate the calendar grid with your keyboard.
 *
 * @nativeInput Use the `input` event to listen to content being modified by the user. It is emitted everytime a user writes or removes content in the input.
 * @nativeChange The `change` event is emitted when the input loses focus, the user clicks `Enter` or makes a selection (such as auto complete or suggestions).
 *
 * @slot chips - Introduce some quick date selectors underneath the calendar grid. Use the `pn-choice-chip` component.
 * @slot helpertext - You can use this slot instead of the prop `helpertext`. Recommended, only if you need to include additional HTML markup. Such as a `pn-text-link`. Use a `span` element to wrap the text and link.
 * @slot error - You can use this slot instead of the prop `error`. Recommended, only if you need to include additional HTML markup. Such as a `pn-text-link`. Use a `span` element to wrap the text and link.
 */
export declare class PnDatePicker {
    id: string;
    idFrom: string;
    idTo: string;
    idFromButton: string;
    idToButton: string;
    idHelper: string;
    idError: string;
    idCalendar: string;
    calendarElement: HTMLDivElement;
    today: Dayjs;
    animation: Animation;
    separators: {
        index: number;
        name: string;
    }[];
    separatorRegex: RegExp;
    listMonths: number[];
    listWeek: number[];
    hostElement: HTMLElement;
    open: boolean;
    openUp: boolean;
    selectingTo: boolean;
    grid: GridDate;
    viewYearStart: number;
    dateViewYear: number;
    dateViewMonth: number;
    dateViewDate: number;
    showHelperSlot: boolean;
    showErrorSlot: boolean;
    isClosing: boolean;
    isExpanding: boolean;
    /** Set a label for the from date. */
    labelFrom: string;
    /** Set a label for the to date. @see {@link range} */
    labelTo?: string;
    /** Provide a helpertext for the date input. */
    helpertext?: string;
    /** Set a predefined value for the from date. @see {@link format} */
    start: string;
    /**
     * Set a predefined value for the from date.
     *
     * @see {@link range}
     * @see {@link format}
     */
    end: string;
    /**
     * Set the date format of the value.
     *
     * While you can set any date value from the Dayjs documentation,
     * we strongly recommend you pick a simple format that you can also type manually.
     *
     * @see {@link https://day.js.org/docs/en/display/format Day.js format documentation.}
     */
    format: string;
    /** Manually set language; this will be inherited from the topbar. */
    language?: PnLanguages;
    /** Set a custom ID for the calendar. @category HTML attributes */
    dateId: string;
    /** HTML input name @category HTML attributes */
    name?: string;
    /**
     * Placeholder for the input field (defaults to the format prop).
     * @see {@link format}
     * @category HTML attributes
     **/
    placeholder?: string;
    /**
     * Placeholder for end date (defaults to the format prop).
     * @see {@link format}
     * @category HTML attributes
     **/
    endPlaceholder?: string;
    /** Set the input `autocomplete` attribute. @category HTML attributes */
    autocomplete?: string;
    /** Set the input `list` attribute for the first date input. @category HTML attributes */
    list?: string;
    /** Set the input `list` attribute for the second date input. @category HTML attributes */
    listEnd?: string;
    /** Set the HTML pattern prop on the input elements. Make sure it matches the format. @category HTML attributes */
    pattern?: string;
    /** Allow the selection of a date range. @category Features */
    range: boolean;
    /**
     * Set a limit on how many days one may select.
     * By default, you can select an unlimited range.
     *
     * @todo Create a range limit function.
     * @see {@link range}
     *
     * @category Features
     * @hide true
     **/
    rangeLimit?: number;
    /** The calendar grid is shown as default. You can set either `months` or `years` as your first choice. @category Features */
    view: CalendarView;
    /** Make the calendar open upwards by default. Opens downwards if there is not enough space. @category Features */
    calendarUp: boolean;
    /** Show weekend numbers to the left of the calendar grid. @category Features */
    weekNumbers: boolean;
    /** Disable the automatic insertion of separators when typing in the input. @category Features */
    disableTypeAhead: boolean;
    /** Remove the option to select dates on weekends. @category Features */
    disableWeekends: boolean;
    /**
     * Individual dates you want to disable. Use a comma separated string.
     *
     * Remember to use the same format that you have in the `format` prop.
     * @see {@link format}
     * @example "YYYY-MM-DD,YYYY-MM-DD"
     * @category Features
     **/
    disabledDates?: string;
    /** If you use a format with an unknown length, disable the max length. @category Features */
    disableMaxLength?: boolean;
    /** Set the date picker as required. @category State */
    required: boolean;
    /** Set the date picker as readonly. @category State */
    readonly: boolean;
    /** Set the date picker as disabled. @category State */
    disabled: boolean;
    /** Set an error message for the date picker. Overwrites the helpertext if used at the same time. @category Validation */
    error?: string;
    /** Trigger the invalid state without an error message. @category Validation */
    invalid: boolean;
    /**
     * Earliest date possible, this will determine how many years back the date picker will show.
     *
     * Remember to use the same format that you have in the `format` prop.
     * @see {@link format}
     * @example "2024-05-25"
     * @category Min/max date
     **/
    minDate?: string;
    watchMin(): void;
    /**
     * Latest date possible, this will determine how many years forward the date picker will show.
     *
     * Remember to use the same format that you have in the `format` prop.
     * @see {@link format}
     * @example "2024-06-25"
     * @category Min/max date
     **/
    maxDate?: string;
    watchMax(): void;
    watchValue(): CustomEvent<{
        start?: string;
        end?: string;
    }>;
    watchValueTo(): CustomEvent<{
        start?: string;
        end?: string;
    }>;
    handleFormat(): void;
    watchId(): void;
    watchView(): void;
    watchOpen(): void;
    handleMessage(): void;
    handleView(): void;
    /**
     * Use the new `dateSelection`. Its here for compatibility. Will be removed in v8.
     * @deprecated Use the new `dateSelection`. Will be removed in v8.
     **/
    dateselection: EventEmitter<PnDatePickerSelectEvent>;
    /** Emits on valid date selection. Either if the user selects a date in the calendar or writes it manually. */
    dateSelection: EventEmitter<PnDatePickerSelectEvent>;
    emitSelection(): void;
    /** Emitted when an invalid value is set. This can only be done if the user writes in the input itself. */
    dateInvalid: EventEmitter<{
        start?: string;
        end?: string;
    }>;
    /** Emitted when the calendar is toggled. */
    toggleCalendar: EventEmitter<boolean>;
    /** Emmitted when you select a new view. */
    currentView: EventEmitter<CalendarView>;
    /**
     * If the select is open and you resize the window.
     * Remove all css props and disable the animations entierly.
     **/
    handleResize(): void;
    componentWillLoad(): Promise<void>;
    private gridHandler;
    private openGrid;
    private closeGrid;
    private animateGrid;
    private animationFinish;
    private cancelAnimations;
    private globalEvents;
    private addGlobalEventListeners;
    private removeGlobalEventListeners;
    private translate;
    private translateDateText;
    private getRect;
    private toggleGrid;
    private hasHelperText;
    /** If any `error` text is present, either via prop/slot. */
    private hasErrorMessage;
    /** If any `error` is active, either via the prop `invalid` or `error` prop/slot. */
    private hasError;
    private checkSlottedHelper;
    private checkSlottedError;
    private viewingCalendar;
    private viewingMonth;
    private viewingYears;
    private viewType;
    private isBeforeMax;
    private isAfterMin;
    private isDisabledWeekend;
    private isDisabledDate;
    private isDisabled;
    private updateGrid;
    private getCurrentViewDate;
    private isSelected;
    private isToday;
    private getCurrentDateObject;
    /** Handle keyboard navigation in the calendar grid. */
    private calendarKeyboardNavigation;
    private navDirection;
    private getYearGrid;
    /** Defaults to the calendar view. */
    private setView;
    private setNavView;
    private setViewYear;
    private setViewMonth;
    private setViewDate;
    private handleSeparator;
    private inputHandler;
    private setValue;
    private getDayAttributes;
    /** Focus the button toggling the calendar. Handles the start/from date on its own. */
    private focusToggleCalendarButton;
    private focusCalendar;
    private resetFocus;
    private handleButtonBlur;
    /**
     * This function queries all tabbable elements inside the calendar popup.
     * Since we allow slotted content it important that we have a function that takes all elements into account.
     * With the `first` and `grid` argument, you can decide which one you want to get.
     * There are fallbacks so you should never get an empty list of elements.
     */
    private calendarTabElement;
    /**
     * We need to listen to the `Esc` and `Tab` key for the entire calendar popup.
     * Regardless if you focus the grid, the nav buttons or slotted content,
     * the popup will close if you press `Esc`. We also need to reset the focus when the user tabs.
     */
    private handleCalendarTabEsc;
    private ariaDescribedby;
    /** Renders the date calendar grid.  */
    private renderDateGrid;
    /** Renders the month calendar grid.  */
    private renderMonthGrid;
    /** Renders the year calendar grid.  */
    private renderYearGrid;
    private renderInput;
    render(): any;
}
