import { EventEmitter } from '../../../stencil-public-runtime';
import type { PnLanguages, PnMultiselectOption } from "../../../index";
/**
 * The `pn-multiselect` gets its options via javascript.
 *
 * Native HTML does not accept arrays of objects. Most frameworks solves this automatically (Vue, react, etc...),
 * but keep this in mind if you use this component outside of a framework environment.
 *
 * @nativeChange The `pn-multiselect` is built with `input[type=checkbox]` elements, so the `change` event works. However, we recommend the customEvents described above.
 *
 * @see {@link https://portal.postnord.com/web-components/?path=/docs/components-input-multiselect--docs#pn-multiselect%20(options) Options documentation}
 */
export declare class PnMultiselect {
    id: string;
    idLegend: string;
    idButton: string;
    idOptions: string;
    idHelper: string;
    idChips: string;
    idSr: string;
    elGroup: HTMLDivElement;
    elButton: HTMLButtonElement;
    elInput: HTMLInputElement;
    elList: HTMLUListElement;
    elChips: HTMLUListElement;
    movingTimer: NodeJS.Timeout;
    srTimer: NodeJS.Timeout;
    hostElement: HTMLElement;
    isSearching: boolean;
    /** Layout state. */
    open: boolean;
    moving: boolean;
    upwards: boolean;
    srMessage?: string;
    /** Label placed above the select */
    label: string;
    /** Display a helper text underneath the select */
    helpertext?: string;
    /**
     * This is what will be shown on load if no value is used.
     * The `placeholder` will override the default text used if you have the `search` prop active.
     */
    placeholder?: string;
    /** HTML name of the checkbox elements. Used for each checkbox inside the multiselect. */
    name?: string;
    /** Select HTML id */
    selectId: string;
    /** HTML name of the input element.*/
    selectName?: string;
    /** Display an icon to the left of the select input */
    icon?: string;
    /** Manually set the language, not needed if you have the pnTopbar available */
    language?: PnLanguages;
    /**
     * Array of options.
     * @hide true
     **/
    options: PnMultiselectOption[];
    /**
     * Adds a "Select all" option into the list.
     * If you use the search feature at the same time, clicking this option will only toggle the options found.
     * @see {@link search}
     * @category Features
     **/
    selectAll: boolean;
    /**
     * Set a custom value for the "Select all value" option.
     * @see {@link selectAll}
     * @category Features
     */
    allValue?: string;
    /**
     * Allow the user to search among the options.
     * The selected options will now display underneath the multiselect element.
     *
     * @category Features
     **/
    search?: boolean;
    /** Set the search query of the multiselect.
     *
     * @see {@link search}
     * @category Features
     */
    searchQuery?: string;
    /**
     * Decide how many items should be shown before ellipsis. Requires the `search` prop to work.
     *
     * @see {@link search}
     * @category Features
     **/
    itemCount: number;
    /**
     * Use a custom max-height for the dropdown list.
     * @description The component will automatically set the max height of the dropdown list on its own.
     * It takes the `pn-topbar` into account and will open in the direction that fits best.
     * Use this prop to override this behaviour and use a custom max-height.
     * @category Features
     **/
    maxHeight?: string;
    /**
     * Force the dropdown to always open upwards.
     * @category Features
     **/
    top?: boolean;
    /**
     * Force the dropdown to always open downwards.
     * @category Features
     **/
    bottom?: boolean;
    /** Set the select as required. @category Validation */
    required?: boolean;
    /** Disable the select. @category Validation */
    disabled?: boolean;
    /** Trigger the invalid state. @category Validation */
    invalid?: boolean;
    /** Display an error message and trigger the invalid state. @category Validation */
    error?: string;
    handleOpen(): void;
    handleSearch(): void;
    handleSearchQuery(): void;
    handleSelectId(): void;
    /**
     * If the select is open and you resize the window.
     * Remove all css props and disable the animations entierly.
     **/
    handleResize({ type }: Event): void;
    /** Dispatched everytime the multiselect is opened or closed. */
    toggleSelect: EventEmitter<{
        open: boolean;
    }>;
    /** This event contains the entire options array with the new props. Dispatched everytime you make changes to any option. */
    allOptions: EventEmitter<PnMultiselectOption[]>;
    /** Dispatched when you toggle an option. Includes all the props of the option. */
    selectedOption: EventEmitter<PnMultiselectOption>;
    /**
     * This event is dispatched when the user toggles the "Choose all options" box.
     * Also triggers when you click the "Select {number} options" if you are performing a search at the same time.
     **/
    selectedAllOptions: EventEmitter<{
        checked: boolean;
        searching: boolean;
        query?: string;
        found?: PnMultiselectOption[];
    }>;
    /**
     * Contains the search text and the options found for that query.
     * Dispacthed everytime you change the search query.
     **/
    searchInput: EventEmitter<{
        query: string;
        found?: PnMultiselectOption[];
    }>;
    componentWillLoad(): Promise<void>;
    componentDidLoad(): void;
    private emitEvents;
    private optionsChecked;
    private optionsTotal;
    private optionsIndex;
    private optionsCheckedTotal;
    private optionsCheckedPreview;
    private optionsCheckedLabels;
    private optionsSearch;
    private noResults;
    private isIndeterminate;
    private optionSelect;
    private optionSelectAll;
    private optionSelected;
    private translate;
    private getRect;
    private ripple;
    private handleMoving;
    private handleSrMesssage;
    private handleDirection;
    private handleMaxHeight;
    private handleWidth;
    private setFocus;
    private hasError;
    private hasMessage;
    private toggleOpen;
    private handleInputKeyboard;
    private setSearchQuery;
    private handleInputSearch;
    private getListItem;
    private getOptionIndex;
    private checkboxNav;
    private handleBlur;
    private handleLabel;
    /** Check if there are no options in the list. */
    private isEmpty;
    private showSelectAll;
    /** Check if the empty/nothing found should be visible. */
    private showEmptyOption;
    private getPlaceholder;
    private handleChange;
    /** Display the "X more selected options" text. */
    private additonalOptions;
    private describedBy;
    private renderOption;
    private renderOptions;
    private renderChips;
    render(): any;
}
